//-----------------------------------------------------------------------
//--  Composant : TImgBtn   version 1.4   -------  Spatul@hotmail.com  --
//-----------------------------------------------------------------------
//--  Statut des images dans le Glyph :                                --
//--  x x x x x x x x x x x x x x x x x x x x x x x x x                --
//--  x           x           x           x           x                --
//--  x  normal   x  survol   x  press   x  disable  x                --
//--  x           x           x           x           x                --
//--  x x x x x x x x x x x x x x x x x x x x x x x x x                --
//-----------------------------------------------------------------------
//--  29/01/07  Rsolution d'un bug graphique d  la variable         --
//--            "ImgValeur" qui doit tre en "private" et non en "var" --
//--            et correction d'un bug mineur (voir CursOver)          --
//--  11/03/04  Ajout de Anchors, Action...                            --
//--            Gestion de l'tat graphique "Enabled"                  --
//--  10/03/04  Gestion de la couleur de trasparence                   --
//--  09/03/04  Rsolution du redessinement des bitmaps                --
//--  08/03/04  Problmes de canvas par rapport  un bitmap            --
//--  06/01/10  Suppression de toute rfrence  la variable "Gris"    --
//              Possible bug du  l'appel de la fonction Invalidate    --
//                         dans la procedure Paint                     --
//-----------------------------------------------------------------------

unit ImgButton;

interface

uses
  Forms, Windows, Messages, SysUtils, Classes, Controls, Graphics, ExtCtrls ;

type
  TImgBtn = class(TCustomControl)
  private
    { Dclarations prives }
    ImgValeur: integer; // 1 up - 2 down - 3 hot   (par dfaut 0:disabled)
    {ImgValeur est plac en priv pour viter de nuire aux autres composants
     du mme type sur d'autres fiches affiches par le OnClick...}
    FOnMouseEnter:TNotifyEvent;   // pour grer le...
    FOnMouseLeave:TNotifyEvent;   // ...passage de la souris
    procedure IsoleBtn; // isoler chaque bouton
  protected
    { Dclarations protges }
    HotImg, DisaImg, UpImg, DownImg: TBitmap; // les 3 stades
    TempImg, GlyphImg: TBitmap;  // l'image temporaire
    procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X, Y :integer); override;
    procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X, Y :integer); override;
    procedure MouseEnter(var Msg:TMessage); message CM_MOUSEENTER;  // pour grer le...
    procedure MouseLeave(var Msg:TMessage); message CM_MOUSELEAVE;  // ...passage de la souris
    procedure SetColor(value:TColor); // rcupration de la couleur de transparence
    procedure SetGlyph(value:TBitmap); // rcupration du Glyph
    procedure SetTransparent(value:Boolean); // rcupration de la transparence
    procedure SetEnabled(Value:Boolean); override;
  public
    { Dclarations publiques }
    FColor: TColor;
    FTransparent: Boolean;
    Constructor Create(AOwner:TComponent); override; // cration du composant
    Destructor Destroy; override; // destruction du composant
    procedure Paint; override; // redessine le composant
    property Owner;
    Property Parent;
  published
    { Dclarations publies }
    property Glyph:TBitmap read GlyphImg write SetGlyph;
    property TransparentColor: TColor read FColor write SetColor;  // couleur de transparence
    property Transparent:Boolean read FTransparent write SetTransparent; // transparance
    property Visible;
    property Enabled;
    property ParentShowHint;
    property ShowHint;
    property Anchors;
    property Action;
    property PopupMenu;
    property OnClick;
    property OnContextPopup;
    property OnMouseDown;   // afficher l'image DownImg
    property OnMouseUp;     // afficher l'image UpImg (normale)
    property OnMouseMove;
    property OnMouseEnter:TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave:TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

procedure Register;

var
 CursOver: boolean;  //voir MouseEnter, MouseLeave et MouseUp
 {CursOver permet d'eviter un bug de statut du bouton si l'utilisateur...
  ...sort ducomposant en gardant le bouton gauche de la souris enfonc}
 //Gris: boolean;
 Hg, Lg: integer; // taille du bouton

implementation
{$R ImgBtn.dcr} // fichier ressource du composant contenant le logo de TImgBtn

procedure Register;
begin
  RegisterComponents('Exemples', [TImgBtn]);
end;

//------ Cration du composant ------
Constructor TImgBtn.Create(AOwner:TComponent);
begin
 Inherited Create(AOwner);
 Height:=25;
 Width:=25;
 Hg:=25;
 Lg:=25;
 Cursor:=crDefault; //HandPoint;   {par dfaut, mets le curseur en forme de main}
 HotImg:=TBitmap.Create;
 DisaImg:=TBitmap.Create;
 TempImg:=TBitmap.Create;
 UpImg:=TBitmap.Create;
 DownImg:=TBitmap.Create;
 GlyphImg:=TBitmap.Create;
 ImgValeur:=1; // par dfault UpImg
 //Gris:=False; // enabled
 DoubleBuffered:=True;
 FColor:=clFuchsia; // couleur de transparence par dfault : Fuchia
 FTransparent:=True; // par dfaut, active la transparence du composant
end;

//-- Destruction du composant  la fermeture --
Destructor TImgBtn.Destroy;
begin
 HotImg.Free;
 DisaImg.Free;
 TempImg.Free;
 UpImg.Free;
 DownImg.Free;
 GlyphImg.Free;
 Inherited Destroy;
end;

//------ rcuprer les images ------
Procedure TImgBtn.SetGlyph(value:TBitmap);
begin
 GlyphImg.Assign(value);
 Hg:=GlyphImg.Height;
 Lg:=GlyphImg.Width div 4;
 Height:=Hg;
 Width:=Lg;
 FColor:=GlyphImg.Canvas.Pixels[0,0]; // prend la couleur de transparence
 IsoleBtn;
end;

//------ Recharger les bitmaps ------
procedure TImgBtn.IsoleBtn; {pour isoler chaque bouton}
var
 RectImg, MyRect: TRect;
begin
 RectImg:=Rect(0,0,Lg,Hg);
 UpImg.Width:=Lg;
 UpImg.Height:=Hg;
 UpImg.Width:=Lg;
 HotImg.Height:=Hg;
 HotImg.Width:=Lg;
 DownImg.Height:=Hg;
 DownImg.Width:=Lg;
 DisaImg.Height:=Hg;
 DisaImg.Width:=Lg;
 MyRect:=Rect(0,0,Lg,Hg);
 UpImg.Canvas.CopyRect(RectImg,GlyphImg.Canvas,MyRect);     // Normal
 MyRect:=Rect(Lg,0,Lg*2,Hg);
 HotImg.Canvas.CopyRect(RectImg,GlyphImg.Canvas,MyRect);    // Survol
 MyRect:=Rect(Lg*2,0,Lg*3,Hg);
 DownImg.Canvas.CopyRect(RectImg,GlyphImg.Canvas,MyRect);   // Press
 MyRect:=Rect(Lg*3,0,Lg*4,Hg);
 DisaImg.Canvas.CopyRect(RectImg,GlyphImg.Canvas,MyRect);   // Disabled
 Height:=Hg;
 Width:=Lg;
end;

//------ Redessiner le composant ------
Procedure TImgBtn.Paint;
begin
 If (UpImg=nil) then exit; // si il n'y a pas d'image... SORT !
 If csDesigning In ComponentState then begin
  // on est en mode cration
  If GlyphImg.Height<>0 then SetGlyph(GlyphImg) else IsoleBtn; {parade pour recharger les bitmap A AMELIORER PAR LA SUITE }
  TempImg.Assign(UpImg);
  TempImg.TransparentColor:=FColor; // prend la couleur de transparence
   If FTransparent=True then TempImg.Transparent:=True else TempImg.Transparent:=False;
  Canvas.Draw(0,0,TempImg);
  Canvas.Pen.Style:=psDot;  // dessine un cadre pour savoir o se trouve le composant
  Canvas.Brush.Style:=bsClear;
  Canvas.Rectangle(0,0,Width,Height);
 end else begin
  // on est en mode excution
  SetGlyph(GlyphImg); {parade pour recharger les bitmap A AMELIORER PAR LA SUITE }
  If (enabled) then begin
   if ImgValeur=1 then TempImg.Assign(UpImg);
   if ImgValeur=2 then TempImg.Assign(DownImg);
   if ImgValeur=3 then TempImg.Assign(HotImg);
   TempImg.TransparentColor:=FColor;  // prend la couleur de transparence
   If FTransparent=True then TempImg.Transparent:=True else TempImg.Transparent:=False;
   Canvas.Draw(0,0,TempImg);
   //If Gris=True then begin
   // Invalidate; {redessine tout le composant}
   // Gris:=False;
   //end;
  end else begin
   TempImg.Assign(DisaImg);
   TempImg.TransparentColor:=FColor; // prend la couleur de transparence
   If FTransparent=True then TempImg.Transparent:=True else TempImg.Transparent:=False;
   Canvas.Draw(0,0,TempImg);
   //If Gris=False then begin
   // Invalidate; {redessine tout le composant}
   // Gris:=True;
   //end;
  end;
 end;
end;

//---- Rcupration de la couleur de transparence ----
procedure TImgBtn.SetColor(value:TColor);
begin
 FColor:=value;
 Invalidate; {redessine tout le composant}
end;

//---- Gre la transparence du composant ----
procedure TImgBtn.SetTransparent(value:Boolean);
begin
 If FTransparent<>value then FTransparent:=value;
 Invalidate; {redessine tout le composant}
end;

//---- Gestion de l'tat graphique "Enabled" ----
procedure TImgBtn.SetEnabled(Value:Boolean);
begin
 Inherited SetEnabled(Value);
 paint;
end;

//-- Lorsque l'on clique sur le composant --
procedure TImgBtn.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:integer);
begin
 Inherited MouseDown(Button,Shift,X,Y);
 if (Button = mbLeft) then
 begin
  ImgValeur:=2;
  paint;
 end;
end;

procedure TImgBtn.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:integer);
begin
 Inherited MouseUp(Button,Shift,X,Y);
 if (Button = mbLeft) and CursOver then
 begin
  ImgValeur:=3; //1
  paint;
 end else begin
  ImgValeur:=1;
  paint;
 end;
end;

//-- Pour grer le passage de la souris au dessus du composant --
procedure TImgBtn.MouseEnter(var Msg: TMessage);
begin
  if Enabled
  then begin
    CursOver:=True;
    ImgValeur:=3;
    paint;
    Invalidate;
    if Assigned(FOnMouseEnter)
    then FOnMouseEnter(Self);
  end;
end;

procedure TImgBtn.MouseLeave(var Msg: TMessage);
begin
  if Enabled then
  begin
   CursOver:=False;
   ImgValeur:=1;
   paint;
   Invalidate;
   if Assigned(FOnMouseLeave)
   then FOnMouseLeave(Self);
  end;
end;

end.
