//--------------------------------------------------------------------
//-- Composant : TPngBtn    version 1.1   ----  Spatul@hotmail.com  --
//-- Ncessite le composant TPNGImage de Gustavo Huffenbacher Daud  --
//-- Tlchargeable  l'adresse : http://pngdelphi.sourceforge.net  --
//--------------------------------------------------------------------
//--  Statut des images dans le PngImage :                          --
//--  x x x x x x x x x x x x x x x x x x x x x x x x x             --
//--  x           x           x           x           x             --
//--  x  normal   x  survol   x  press   x  disable  x             --
//--  x           x           x           x           x             --
//--  x x x x x x x x x x x x x x x x x x x x x x x x x             --
//--------------------------------------------------------------------
//--  30/06/05  Lors de l'appel  REFRESH, le canvas du composant   --
//--            se redessine en effaant l'ancien PNG.              --
//--  29/06/05  Cration du composant de base.                      --
//--  02/07/05  Ajout de la proprit "Down" qui si Actif affiche   --
//--            le 3me tat du bouton                              --
//--------------------------------------------------------------------

Unit PngButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, ExtCtrls;

type
  TPngBtn = class(TGraphicControl)
  private
    {Dclarations Prives}
    FOnMouseEnter:TNotifyEvent;   // pour grer le...
    FOnMouseLeave:TNotifyEvent;   // ...passage de la souris
  protected
    {Dclarations Protges}
    EtatBtn: Integer; // 0=Normal 1=Survol 2=Press 3=Disabled
    procedure SetPngImg(value:TPngObject); // rcupration du Glyph
    procedure SetDown(value:boolean); // tat press actif
    procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X, Y :integer); override;
    procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X, Y :integer); override;
    procedure MouseEnter(var Msg:TMessage); message CM_MOUSEENTER;  // pour grer le...
    procedure MouseLeave(var Msg:TMessage); message CM_MOUSELEAVE;  // ...passage de la souris
    procedure SetEnabled(Value:Boolean); override;
  public
    {Dclarations Publiques}
    FPngImg : TBitmap; //Conteneur graphique des 4 tats de l'image
    FDown: Boolean;   // option "press" dfinit par l'utilisateur
    Constructor Create(AOwner:TComponent); override; // cration du composant
    Destructor Destroy; override; // destruction du composant
    procedure Paint; override; // redessine le composant
  published
   {Dclarations Publies}
   property PngImg:TBitmap read FPngImg write SetPngImg;
   property Down:Boolean read FDown write SetDown default False;
   property Action;
   property Anchors;
   property Enabled;
   property ParentShowHint;
   property PopupMenu;
   property ShowHint;
   property Visible;
   property OnClick;
   property OnContextPopup;
   property OnMouseDown;   // afficher l'image DownImg
   property OnMouseUp;     // afficher l'image UpImg (normale)
   property OnMouseMove;
   property OnMouseEnter:TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
   property OnMouseLeave:TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

procedure Register;

implementation
{$R PngBtn.dcr} // fichier ressource du composant contenant le logo de TPngButton 

procedure Register;
begin
  RegisterComponents('Exemples', [TPngBtn]);
end;

//------ Cration du composant ------
Constructor TPngBtn.Create(AOwner:TComponent);
begin
 Inherited Create(AOwner);
 Height:=25;
 Width:=75;
 FPngImg:=TBitmap.Create;
end;

//-- Destruction du composant  la fermeture --
Destructor TPngBtn.Destroy;
begin
 FPngImg.Free;
 Inherited Destroy;
end;

//------ rcuprer les images ------
Procedure TPngBtn.SetPngImg(value:TPngObject);
begin
 FPngImg.Assign(value);
 {mise  jour de la taille du compo par rapport  l'image}
 Height:=FPngImg.Height;
 Width:=FPngImg.Width div 4;
 {bloquer le changement de taille du compo  la taille de l'image PNG}
 With Constraints do begin
  MaxHeight:=Height;
  MinHeight:=Height;
  MaxWidth:=Width;
  MinWidth:=Width;
 end;
 EtatBtn:=0; {mettre l'tat du bouton  "normal"}
end;

//------ Mise  jour de l'tat du bouton : Down ? ------
procedure TPngBtn.SetDown(value:boolean);
begin
  FDown:=value;
  if FDown=False then EtatBtn:=0; {mettre l'tat du bouton  "normal"}
  paint;
  Refresh; //permet d'ffacer le canvas et de redessiner dessus (pour grer la transparence)
end;

//------ Redessiner le composant ------
Procedure TPngBtn.Paint;
begin
 If csDesigning In ComponentState then
 begin // on est en mode conception
  {bloquer le changement de taille du compo si il contient dja une image PNG}
  If FPngImg<>nil then begin
   With Constraints do begin
    MaxHeight:=FPngImg.Height;
    MinHeight:=FPngImg.Height;
    MaxWidth:=FPngImg.Width div 4;
    MinWidth:=FPngImg.Width div 4;
   end;
  end;
  {grer l'affichage du compo en mode conception : Enabled / disabled ou Down}
  If (FPngImg<>nil) and (enabled) then FPngImg.Draw(Canvas,Rect(0,0,width,Height)); // Normal
  If (FPngImg<>nil) and (enabled) and (FDown=True) then FPngImg.Draw(Canvas,Rect(-(Width*2)-1,0,width,Height)); // Press
  If (FPngImg<>nil) and not(enabled) then FPngImg.Draw(Canvas,Rect(-(Width*3)-1,0,width,Height)); // Disabled
  {dessine un cadre pour savoir o se trouve le composant}
  Canvas.Pen.Style:=psDot;
  Canvas.Brush.Style:=bsClear;
  Canvas.Rectangle(0,0,Width,Height);
 end else begin // on est en mode excution
   {si le composant est Down ou Disabled, lui attribuer l'tat correspondant}
   If (FDown=True) then EtatBtn:=2;
   If not(enabled) then EtatBtn:=3;
   {dessiner l'image PNG en fonction de l'tat du bouton}
   Case EtatBtn of
    0: FPngImg.Draw(Canvas,Rect(0,0,width,Height)); // Normal
    1: FPngImg.Draw(Canvas,Rect(-Width-1,0,width,Height)); // Survol
    2: FPngImg.Draw(Canvas,Rect(-(Width*2)-1,0,width,Height)); // Press
    3: FPngImg.Draw(Canvas,Rect(-(Width*3)-1,0,width,Height)); // Disabled
   end;
 end;
end;

//---- Gestion de l'tat graphique "Enabled" ----
procedure TPngBtn.SetEnabled(Value:Boolean);
begin
 Inherited SetEnabled(Value);
 If (enabled) then EtatBtn:=0 else EtatBtn:=3;
 paint;
end;

//---- Lorsque l'on clique sur le composant ----
procedure TPngBtn.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:integer);
begin
 Inherited MouseDown(Button,Shift,X,Y);
 if (Button = mbLeft) and (enabled) then
 begin
  EtatBtn:=2; // press
  paint;
  Refresh; //permet d'ffacer le canvas et de redessiner dessus (pour grer la transparence)
 end;
end;

//---- Lorsque l'on lache le click ----
procedure TPngBtn.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:integer);
begin
 Inherited MouseUp(Button,Shift,X,Y);
 if (Button = mbLeft) and (enabled) then
 begin
  If (X in [0..Width]) and (Y in [0..Height])
      then EtatBtn:=1 {Survol} else EtatBtn:=0 {normal};
  paint;
  Refresh; //permet d'ffacer le canvas et de redessiner dessus (pour grer la transparence)
 end;
end;

//-- Pour grer le passage de la souris au dessus du composant --
//-- Quand la souris survole le composant... --
procedure TPngBtn.MouseEnter(var Msg: TMessage);
begin
  if Enabled and not(csDesigning In ComponentState)
  then begin
   EtatBtn:=1; // Survol
   paint;
   Refresh; //permet d'ffacer le canvas et de redessiner dessus (pour grer la transparence)
  end;
  if Assigned(FOnMouseEnter)
  then FOnMouseEnter(Self);
end;

//-- Quand la souris quitte le composant... --
procedure TPngBtn.MouseLeave(var Msg: TMessage);
begin
 //on travaille en mode excution et enabled
 if Enabled and not(csDesigning In ComponentState) then
  begin
   EtatBtn:=0; //Normal
   paint;
   Refresh; //permet d'ffacer le canvas et de redessiner dessus (pour grer la transparence)
  end;
  if Assigned(FOnMouseLeave)
  then FOnMouseLeave(Self);
end;


end.
