// BASSCD Test, copyright (c) 2003-2006 Ian Luck.

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <commctrl.h>
#include "basscd.h"

HWND win=NULL;

#define MAXDRIVES 10
int curdrive=0;
HSTREAM stream[MAXDRIVES]={0};

// Display error dialogs
void Error(const char *es)
{
	char mes[200];
	sprintf(mes,"%s\n(error code: %d)",es,BASS_ErrorGetCode());
	MessageBox(win,mes,0,0);
}

// Messaging macro
#define MESS(id,m,w,l) SendDlgItemMessage(win,id,m,(WPARAM)w,(LPARAM)l)

void PlayTrack(DWORD drive, DWORD track);

// End sync
void CALLBACK EndSync(HSYNC handle, DWORD channel, DWORD data, DWORD user)
{
	if (MESS(13,BM_GETCHECK,0,0)) { // advance onto next track
		DWORD track=BASS_CD_StreamGetTrack(channel);
		DWORD drive=HIWORD(track);
		track=(LOWORD(track)+1)%BASS_CD_GetTracks(drive);
		if (drive==curdrive) MESS(11,LB_SETCURSEL,track,0);
		PlayTrack(drive,track);
	}
}

void PlayTrack(DWORD drive, DWORD track)
{
	if (stream[drive])
		BASS_CD_StreamSetTrack(stream[drive],track); // already have a stream, so just set the track
	else {
		stream[drive]=BASS_CD_StreamCreate(drive,track,0); // create stream
		BASS_ChannelSetSync(stream[drive],BASS_SYNC_END,0,&EndSync,0); // set end sync
	}
	if (drive==curdrive)
		MESS(30,TBM_SETRANGEMAX,0,BASS_ChannelGetLength(stream[drive])/176400); // set pos scroller range
	BASS_ChannelPlay(stream[drive],FALSE); // start playing
}

void UpdateTrackList()
{
	char text[100],*cdtext;
	int a,tc=BASS_CD_GetTracks(curdrive);
	MESS(11,LB_RESETCONTENT,0,0);
	if (tc==-1) return; // no CD
	cdtext=BASS_CD_GetID(curdrive,BASS_CDID_TEXT); // get CD-TEXT
	for (a=0;a<tc;a++) {
		int l=BASS_CD_GetTrackLength(curdrive,a);
		sprintf(text,"Track %02d",a+1);
		if (cdtext) {
			char *t=cdtext,tag[10];
			sprintf(tag,"TITLE%d=",a+1); // the CD-TEXT tag to look for
			while (*t) {
				if (!memcmp(t,tag,strlen(tag))) { // found the track title...
					strcpy(text,t+strlen(tag)); // replace "track x" with title
					break;
				}
				t+=strlen(t)+1;
			}
		}
		if (l==-1)
			strcat(text," (data)");
		else {
			l/=176400;
			sprintf(text+strlen(text)," (%d:%02d)",l/60,l%60);
		}
		MESS(11,LB_ADDSTRING,0,text);
	}
	a=BASS_CD_StreamGetTrack(stream[curdrive]);
	if (a!=-1) { // this drive has a stream
		MESS(11,LB_SETCURSEL,LOWORD(a),0); // select current track
		MESS(30,TBM_SETRANGEMAX,0,BASS_ChannelGetLength(stream[curdrive])/176400); // set pos scroller range
	}
	{
		DWORD vol=100,spd=44100;
		BASS_ChannelGetAttributes(stream[curdrive],&spd,&vol,NULL);
		MESS(31,TBM_SETPOS,TRUE,100-vol); // set volume slider pos
		MESS(32,TBM_SETPOS,TRUE,spd/441); // set speed slider pos
		sprintf(text,"Speed - %d%%",spd/441);
		MESS(50,WM_SETTEXT,0,text);
	}
}

BOOL CALLBACK dialogproc(HWND h,UINT m,WPARAM w,LPARAM l)
{
	static int seeking=-1;
	switch (m) {
		case WM_TIMER:
			{
				// update levels
				static int updatecount=0,levl=0,levr=0;
				int level=BASS_ChannelGetLevel(stream[curdrive]);
				levl-=1500; if (levl<0) levl=0;
				levr-=1500; if (levr<0) levr=0;
				if (level!=-1) {
					if (levl<LOWORD(level)) levl=LOWORD(level);
					if (levr<HIWORD(level)) levr=HIWORD(level);
				}
				MESS(40,PBM_SETPOS,levl,0); // left
				MESS(41,PBM_SETPOS,levr,0); // right
				if (!(updatecount++&3)) { // do other stuff (only every 4th visit)
					char time[20]="-";
					BOOL isopen=BASS_CD_DoorIsOpen(curdrive);
					BOOL islock=BASS_CD_DoorIsLocked(curdrive);
					MESS(14,BM_SETCHECK,isopen,0);
					MESS(15,BM_SETCHECK,islock,0);
					if (BASS_ChannelIsActive(stream[curdrive])) { // playing - update info
						DWORD p=seeking;
						if (p==-1) { // not seeking - update pos scroller
							p=BASS_ChannelBytes2Seconds(stream[curdrive],BASS_ChannelGetPosition(stream[curdrive]));
							MESS(30,TBM_SETPOS,TRUE,p);
						}
						sprintf(time,"%d - %d:%02d",LOWORD(BASS_CD_StreamGetTrack(stream[curdrive]))+1,p/60,p%60);
					} else if (!MESS(11,LB_GETCOUNT,0,0)) { // empty track list - refresh
						if (!isopen) UpdateTrackList();
					} else if (isopen || !BASS_CD_IsReady(curdrive)) { // no CD - free stream & clear list
						BASS_StreamFree(stream[curdrive]);
						stream[curdrive]=0;
						MESS(11,LB_RESETCONTENT,0,0);
					}
					MESS(22,WM_SETTEXT,0,time);
				}
			}
			break;

		case WM_COMMAND:
			switch (LOWORD(w)) {
				case IDCANCEL:
					EndDialog(h,0);
					return 1;
				case 10:
					if (HIWORD(w)==CBN_SELCHANGE) { // change current drive
						curdrive=MESS(10,CB_GETCURSEL,0,0);
						UpdateTrackList();
					}
					return 1;
				case 11:
					if (HIWORD(w)==LBN_SELCHANGE) { // change playing track
						DWORD track=MESS(11,LB_GETCURSEL,0,0);
						PlayTrack(curdrive,track);
					}
					return 1;
				case 12: // play/pause
					if (BASS_ChannelIsActive(stream[curdrive])==BASS_ACTIVE_PLAYING)
						BASS_ChannelPause(stream[curdrive]);
					else
						BASS_ChannelPlay(stream[curdrive],FALSE);
					return 1;
				case 14: // open/close door
					BASS_CD_Door(curdrive,BASS_CD_DoorIsOpen(curdrive)?BASS_CD_DOOR_CLOSE:BASS_CD_DOOR_OPEN);
					break;
				case 15: // lock/unlock door
					BASS_CD_Door(curdrive,BASS_CD_DoorIsLocked(curdrive)?BASS_CD_DOOR_UNLOCK:BASS_CD_DOOR_LOCK);
					break;
			}
			break;

		case WM_HSCROLL:
			if (GetDlgCtrlID((HWND)l)==30) { // position scroller
				seeking=MESS(30,TBM_GETPOS,0,0);;
				if (LOWORD(w)==SB_ENDSCROLL) { // seek to new pos
					BASS_ChannelSetPosition(stream[curdrive],seeking*176400);
					seeking=-1;
				}
			} else { // must be speed slider then
				char text[20];
				DWORD spd=MESS(32,TBM_GETPOS,0,0);
				BASS_ChannelSetAttributes(stream[curdrive],spd*441,-1,-101);
				sprintf(text,"Speed - %d%%",spd);
				MESS(50,WM_SETTEXT,0,text);
			}
			return 1;

		case WM_VSCROLL:
			{ // adjust volume
				DWORD vol=MESS(31,TBM_GETPOS,0,0);
				BASS_ChannelSetAttributes(stream[curdrive],-1,100-vol,-101);
			}
			return 1;

		case WM_INITDIALOG:
			win=h;
			{ // Get list of available drives
				int a=0;
				const char *n;
				while (a<MAXDRIVES && (n=BASS_CD_GetDriveDescription(a))) {
					char text[100];
					sprintf(text,"%c: %s",BASS_CD_GetDriveLetter(a)+'A',n); // "letter: description"
					MESS(10,CB_ADDSTRING,0,text);
					a++;
				}
				if (!a) {
					Error("No CD drives found");
					EndDialog(h,0);
					return 0;
				}
				MESS(10,CB_SETCURSEL,0,0); // select 1st drive
			}
			// Setup output - default device
			if (!BASS_Init(-1,44100,0,win,NULL)) {
				Error("Can't initialize device");
				EndDialog(h,0);
				return 0;
			}
			// setup scroller/bar stuff
			MESS(30,TBM_SETLINESIZE,0,5);
			MESS(30,TBM_SETPAGESIZE,0,30);
			MESS(31,TBM_SETRANGEMAX,0,100);
			MESS(31,TBM_SETLINESIZE,0,1);
			MESS(32,TBM_SETRANGEMIN,0,50);
			MESS(32,TBM_SETRANGEMAX,0,150);
			MESS(32,TBM_SETLINESIZE,0,1);
			MESS(32,TBM_SETPAGESIZE,0,10);
			MESS(40,PBM_SETRANGE32,0,30000);
			MESS(41,PBM_SETRANGE32,0,30000);
			SetTimer(h,1,50,NULL);
			return 1;

		case WM_DESTROY:
			KillTimer(h,1);
			BASS_Free();
			return 1;
	}
	return 0;
}

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,LPSTR lpCmdLine, int nCmdShow)
{
	// check the correct BASS was loaded
	if (HIWORD(BASS_GetVersion())!=BASSVERSION) {
		MessageBox(0,"An incorrect version of BASS.DLL was loaded",0,MB_ICONERROR);
		return 0;
	}

	{ // enable trackbar support
		INITCOMMONCONTROLSEX cc={sizeof(cc),ICC_BAR_CLASSES};
		InitCommonControlsEx(&cc);
	}

	// display the window
	DialogBox(hInstance,MAKEINTRESOURCE(1000),NULL,&dialogproc);

	return 0;
}
